********************************************************************************************
*  Written by Aldo Sandoval-Hernandez
*  email:   asandov3@uwo.ca 	
*  download latest version at www.aldosandoval.com 
********************************************************************************************


clear all
set more off

* Directories
global datadir  "Specify directory"
global datavdir "Specify directory"

*									Programs and functions
********************************************************************************************
********************************************************************************************

program bs_resid, eclass
	syntax, RESidual(varname numeric) MATrix(name) CL(name)
	
	*get the varlist
	local xvars : colna  `matrix'
	local CONS _cons
	local xvars : list xvars -CONS
	
	* compute the linear predictor
	tempvar xb  y u vmat
	matrix score double `xb' = `matrix'
	
	* Resample residuals as in the paper
    mata: resample("`cl'")  
	* Create the new dependent variable using resampled residuals
	gen double `y' = `xb' + `residual'*eta
	regress `y' `xvars'
	predict `u', residual
	matrix `vmat' = e(V)
	matrix list `vmat' 
	mata: spatialHAC("`xvars'", "`u'","Vspatial")
	matrix `vmat' = Vspatial 
	matrix list `vmat'
	ereturn repost V = `vmat' 
	drop eta
	
end


program HAC
	syntax, RESidual(varname numeric) MATrix(name)
	
	*get the varlist
	local xvars : colna  `matrix'
	local CONS _cons
	local xvars : list xvars -CONS
	mata: spatialHAC("`xvars'","`residual'" ,"Vspatial2")
end


mata:
	    /******* Function: Gaussian Kernel */
		void function gaussian_kernel_multi(string lon,string lat,string wp,string kl, string kernel, string Lvector)
	{
		X  = st_data(.,lon)
		Y  = st_data(.,lat)
		KL = st_data(.,kl)
		WP = st_data(.,wp)
		
		r = rows(X)
		D = J(r,r,0)
		K = J(r,r,0)
		
		/* Each distance is normalized by the median of bilateral distances */
		dstar = 568.5419012
		tstar = 1.186140465
		
		/*Combined distance*/
		for (i=1; i<=r; i++) {
		for (j=1; j<=i; j++) {
			D[i,j] =  (( (X[i]-X[j] )^2 + (Y[i]-Y[j] )^2)^(1/2))/(dstar*1000) + (( (KL[i]-KL[j] )^2 + (WP[i]-WP[j] )^2)^(1/2))/(tstar)
			D[j,i] = D[i,j]
		}
		}
	
		/* Compute Gaussian Kernel*/
		for (i=1; i<=r; i++) {
		for (j=1; j<=i; j++) {
			K[i,j] = exp(  -(D[i,j])^2 )		
			K[j,i] = K[i,j]
		}
		}
	X=.
	L=.
	
	/* Do the eigendecomposition of the matrix*/
	symeigensystem(K,X,L)
	
	Phi = I(r)
		for (i=1; i<=r; i++) {
			Phi[i,i]=0
			if (L[i] >0)  Phi[i,i] = (L[i])^(1/2) 
			;
		}	
	
	Ln = X*Phi	
	st_matrix(Lvector,Ln)
	st_matrix(kernel,K)
	}
	
	
	/******* Function: Resampling clustered spatial bootstrap */
	void resample(string clust_)
	{	
	X  = st_data(.,clust_)
	E = uniqrows(X)
	nr = rows(E)
	b = st_matrix("Ln")
	r = rows(b)
	no = rnormal(nr,1,0,1)	
	nu = J(r,1,0)	
		for (i=1; i<=nr; i++) {
		for (j=1; j<=r; j++) {
			if ( X[j]==E[i] )  nu[j]= no[i]
			;
		}
		}
	eta = b*nu
	st_addvar("double","eta")
	st_store(.,"eta",eta)
	}
	
	/******* Function: Spatial HAC Estimator of var-cov matrix */
	void spatialHAC(string varnames, string residuals, string varmat)
	{
	X = st_data(.,varnames)
	X = X, J(rows(X),1,1)
	u = st_data(.,residuals)
	n = rows(X)
	rows(X)
	
	A = invsym((X'*X):/n)	
	Khat = st_matrix("K")
	
	XU = X:*u
	Q = J(cols(X),cols(X),0)
	
	for (i=1; i<= n; i++){
		xu  = XU:*(Khat[i,.]')
		XU_ = (XU[i,.]')*colsum(xu)
		Q = Q + XU_
	}
	Q = Q:/n
	V = (A*Q*A):/(n-cols(X))
	st_matrix(varmat,V)

	}
	
	

end



*******************************************************************************************************

* Open NALMF, create same ID and merge
clear
use "$datadir\nalmf_2002_2012_panel.dta"
keep if  year ==2003 | year == 2007

gen sector=substr(naics,1,2)
gen naicsd =substr(naics,1,5) /* 5-digit NAICS*/

* Just focus on manufacturers
keep if sector=="31" | sector=="32" | sector=="33"

* Create id of economic region, first two digits correspond to the province
drop if opaddressprovince==""
drop if economicregion==""
drop if statisticalareaclassification==""
gen PRID =""
replace PRID ="10" if opaddressprovince=="NL"
replace PRID ="11" if opaddressprovince=="PE"
replace PRID ="12" if opaddressprovince=="NS"
replace PRID ="13" if opaddressprovince=="NB"
replace PRID ="24" if opaddressprovince=="QC"
replace PRID ="35" if opaddressprovince=="ON"
replace PRID ="46" if opaddressprovince=="MB"
replace PRID ="47" if opaddressprovince=="SK"
replace PRID ="48" if opaddressprovince=="AB"
replace PRID ="59" if opaddressprovince=="BC"
replace PRID ="60" if opaddressprovince=="YT"
replace PRID ="61" if opaddressprovince=="NT"
replace PRID ="62" if opaddressprovince=="NU"

egen CMAPUID = concat( PRID statisticalareaclassification )
egen ERUID = concat( PRID economicregion )
destring ERUID, gen(geog)


* Total imports by ERUID
merge m:1 ERUID using "$datavdir\Imports_spatial.dta", keepusing(import*)
drop if _merge==2
replace imports=0 if _merge==1
replace import_int=0 if _merge==1
replace import_fin=0 if _merge==1
replace import_cap=0 if _merge==1
drop _merge

* Total Sales
merge m:1 ERUID using "$datavdir\Sales_spatial.dta", keepusing(Dom_Sales)
keep if _merge==3
drop _merge


*************** Sales and import penetration variables
xtset entid year
* Create the same identifier and rename some variables
rename sales_goods_and_services sales
rename total_tangible_assets capital
gen inventory = opening_inventory - closing_inventory

* These are my measures of import penetration
gen import_pen = imports/(imports +Dom_Sales)

gen imports_pen_int = import_int/(imports +Dom_Sales)
gen imports_pen_fin = import_fin/(imports +Dom_Sales)
gen imports_pen_cap = import_cap/(imports +Dom_Sales)

local penetration  " imports_pen_int imports_pen_fin imports_pen_cap"

/******************* interest***************************/
gen ind3     = substr(naics,1,3) /* Price indices are at the 3-digit industry*/
gen yrb = year(birthdate)
gen age  = year-yrb
gen age2 = age^2

* Obtain industry-specific price indexes at the 3-digit industry
egen id_price =concat(ind3 year)
merge m:1 id_price using "$datavdir\industry_price.dta", keepusing(P_*)
drop _merge id_price ind3

********* Compute real variables and log changes ****************
*Sales
gen rsales = sales/P_4 /* P_4 is the correct price index gross output*/
gen lrsales = log(rsales)

* Capital
gen rK     = capital/P_2
gen lrK    = log(rK)
* Labour and labour productivity (rlp)
gen lilu   = log(t4_ilu)
gen rL     = t4_payroll/P_3
gen rlp    = rsales/t4_ilu
gen lrlp   = log(rlp)
* Intermediates
gen M       = sales - t4_payroll- gross_profits
gen rM = M/P_5
gen lrM = log(rM)
* Control variables
gen k_l    = rK/rL
gen K_L    = log(rK/rL)
gen lage   = log(age)
gen W_P    = t4_payroll/direct_wages
label variable K_L "Capital intensity"
label variable W_P "Wagebill ratio"

*************** Time period 2003 to 2007 *****************
sort entid year
* create variable in differences
gen delta_rsales = F4.lrsales - lrsales
keep if year==2003

***************** Controls and fixed effects ******************
local controls "age K_L W_P "
egen  FE=group(entid)
gen industry = substr(naics,1,3)
egen  CL  = group(ERUID) 
destring industry, gen(ind_)
egen  CL2 = group( ind_ )
count

* Get rid of missing values in the variables of interest		   
keep if age!=. & K_L!=. & W_P!=. & delta_rsales !=. & lrsales !=.		   
count

* Get coordinates of the centroids of the economic regions
merge m:1 ERUID using "$datavdir\ecr_d.dta", keepusing(myID  x_centroid y_centroid) 
keep if _merge==3
drop _merge

* Restrict sample to firms with more than 20 labor units
drop if t4_ilu<20 | t4_ilu==.
count

**************** Save regression specific data for vetting purposes
*keep delta_rsales `penetration' lrsales  `controls' year CL CL2 x_centroid y_centroid k_l ind_ geog
saveold "$datavdir\SampleReg.dta", version(13) replace 



***** Estimate regressions, keep the betas and standard errors
* iid standard errors
reg delta_rsales `penetration' lrsales  `controls'
matrix Table = e(b)'
matrix V0 = e(V)
mata: st_matrix("V",sqrt(diagonal(st_matrix("V0"))))
matrix Table = Table,V

* robust standard errors
reg delta_rsales `penetration' lrsales  `controls' ,vce(robust)
matrix V0 = e(V)
mata: st_matrix("V",sqrt(diagonal(st_matrix("V0"))))
matrix Table = Table,V

* clustered ecr standard errors
reg delta_rsales `penetration' lrsales  `controls' ,vce(cluster CL)
matrix V0 = e(V)
mata: st_matrix("V",sqrt(diagonal(st_matrix("V0"))))
matrix Table = Table,V

* clustered (industry) standard errors
reg delta_rsales `penetration' lrsales  `controls' ,vce(cluster CL2)
matrix V0 = e(V)
mata: st_matrix("V",sqrt(diagonal(st_matrix("V0"))))
matrix Table = Table,V
matrix list Table

* Now compute the distance and kernel matrix and eigendecompose it
mata: gaussian_kernel_multi("x_centroid","y_centroid","W_P","k_l","K","Ln")


* Saptial HAC with the Kernel matrix 
reg delta_rsales `penetration' lrsales  `controls' 
matrix b = e(b)
predict double u, residuals
HAC, resid(u) mat(b)
mata: st_matrix("V",sqrt(diagonal(st_matrix("Vspatial2"))))
matrix Table = Table,V
scalar nvar = rowsof(Table)

matrix list Table


*************************** Spatial Wild Bootstrap

preserve
*** Spatial Wild bootsrap 
qui reg delta_rsales `penetration' lrsales  `controls' 
matrix b = e(b)
predict double resid, residuals


simulate _b _se, reps(2000) : bs_resid, resid(resid) mat(b) cl(entid)
sum, separator(8)

* Now compute the 95% interval of the coefficients
local a=1
foreach v of varlist _b_* {
	gen true`v'   = b[1,`a']
	egen p975`v'  = pctile(`v'), p(97.5)
	egen p025`v'  = pctile(`v'), p(2.5)
	gen under`v'  = true`v'-p025`v'
	gen over`v'   = p975`v'-true`v'

	
	loc ++a
}

* Get the t-statistics
mat CV     =J(nvar,1,0)
mat seHAC  =J(nvar,1,0)
local vars  " imports_pen_int imports_pen_fin imports_pen_cap lrsales age K_L W_P cons"
local a=1
foreach v of local vars {
	gen t_`v' = abs((_b_`v'-b[1,`a'])/_se_`v')
	sum t_`v', detail
	mat seHAC[`a',1] = V[`a',1]
	mat CV[`a',1] = r(p95)
	loc ++a
}

mat li CV
mat li seHAC

mat CI  =J(nvar,2,0)


local a=1
foreach v of varlist _b_* {
	mat CI[`a',1] = b[1,`a'] - CV[`a',1]*seHAC[`a',1]
	mat CI[`a',2] = b[1,`a'] + CV[`a',1]*seHAC[`a',1]
	loc ++a
}


matrix Table= Table,CI

restore




*************************


preserve
*** Spatial Wild bootsrap Clustered at the ECR-level
qui reg delta_rsales `penetration' lrsales  `controls' 
matrix b = e(b)
predict double resid, residuals


simulate _b _se, reps(2000) : bs_resid, resid(resid) mat(b) cl(geog)
sum, separator(8)

* Now compute the 95% interval of the coefficients
local a=1
foreach v of varlist _b_* {
	gen true`v'   = b[1,`a']
	egen p975`v'  = pctile(`v'), p(97.5)
	egen p025`v'  = pctile(`v'), p(2.5)
	gen under`v'  = true`v'-p025`v'
	gen over`v'   = p975`v'-true`v'

	
	loc ++a
}

* Get the t-statistics
mat CV     =J(nvar,1,0)
mat seHAC  =J(nvar,1,0)
local vars  " imports_pen_int imports_pen_fin imports_pen_cap lrsales age K_L W_P cons"
local a=1
foreach v of local vars {
	gen t_`v' = abs((_b_`v'-b[1,`a'])/_se_`v')
	sum t_`v', detail
	mat seHAC[`a',1] = V[`a',1]
	mat CV[`a',1] = r(p95)
	loc ++a
}

mat li CV
mat li seHAC

mat CI  =J(nvar,2,0)


local a=1
foreach v of varlist _b_* {
	mat CI[`a',1] = b[1,`a'] - CV[`a',1]*seHAC[`a',1]
	mat CI[`a',2] = b[1,`a'] + CV[`a',1]*seHAC[`a',1]
	loc ++a
}


matrix Table= Table,CI


restore


*** Spatial Wild bootsrap Clustered at the Industry-level
qui reg delta_rsales `penetration' lrsales  `controls' 
matrix b = e(b)
predict double resid, residuals


simulate _b _se, reps(2000) : bs_resid, resid(resid) mat(b) cl(ind_)
sum, separator(8)

* Now compute the 95% interval of the coefficients
local a=1
foreach v of varlist _b_* {
	gen true`v'   = b[1,`a']
	egen p975`v'  = pctile(`v'), p(97.5)
	egen p025`v'  = pctile(`v'), p(2.5)
	gen under`v'  = true`v'-p025`v'
	gen over`v'   = p975`v'-true`v'

	
	loc ++a
}

* Get the t-statistics
mat CV     =J(nvar,1,0)
mat seHAC  =J(nvar,1,0)
local vars  " imports_pen_int imports_pen_fin imports_pen_cap lrsales age K_L W_P cons"
local a=1
foreach v of local vars {
	gen t_`v' = abs((_b_`v'-b[1,`a'])/_se_`v')
	sum t_`v', detail
	mat seHAC[`a',1] = V[`a',1]
	mat CV[`a',1] = r(p95)
	loc ++a
}

mat li CV
mat li seHAC

mat CI  =J(nvar,2,0)


local a=1
foreach v of varlist _b_* {
	mat CI[`a',1] = b[1,`a'] - CV[`a',1]*seHAC[`a',1]
	mat CI[`a',2] = b[1,`a'] + CV[`a',1]*seHAC[`a',1]
	loc ++a
}





matrix Table= Table,CI
mat colnames Table = "beta" "iid" "White" "cl ecr" "cl ind" "HAC" "[95% Conf." "Interval]" "[95% Conf." "Interval]" "[95% Conf." "Interval]"
mat li Table


estout matrix(Table), ///
mlabels(" '                                                                                      SWB                        SWB ECR                    SWB IND    ", span)                                                                                                   

estout matrix(Table) using "Table_spatboot.tex", style(tex) replace ///
mlabels(" '                                                                                      SWB                        SWB ECR                    SWB IND    ", span)         

*log close
