% program for Conley, Goncalves, Kim, and Perron (QE, 2022)
% computes results plotted in Figure 2

clc
clear all

diary basic2.out %output file

randn('state',4587); %initialize seeds to replicate results
rand('state',123);

nbrep = 10000;   % number of replications
B = 399;        % numbre of bootstrap replications

beta0 = 1;      % true parameter value

thetaset = [0:.1:.9]; %values of CS dependence parameter

nset = [25,100,400]; % sample sizes

bandset = [.5:.5:4];

distance = 1 

for nindex = 1:length(nset);
    n = nset(nindex);
    
    % draw set of locations once for all replications
    
    w = sqrt(n); % size of lattice
    locations0 = rand(n,2)*w; % set of n x 2 locariosn between 0 and w
    
colindex = repmat([1:w]',w,1); % horizontal locations        
rowindex= reshape(repmat([1:w]',1,w)',n,1); % vertical locations        

D = zeros(n,n); 
        
        locations1 = locations0 + randn(n,2)*sqrt(2); % add (N0,2)
        
    % compute distances
    
    D0=zeros(n,n);D1=D0;D2=D0;D3=D0;D4=D0; %initialize distance matrices
    
    for i = 1:n;
        
        D0(i,:) = sqrt( (locations0(i,1)-locations0(:,1)).^2 + (locations0(i,2)-locations0(:,2)).^2 );
        if distance >0;
            D1(i,:) = sqrt( (locations1(i,1)-locations1(:,1)).^2 + (locations1(i,2)-locations1(:,2)).^2 );
        end %if
     end; %i
    
      % define neighborhoods for approximating model and  row normalize - max avoids cases with 0 neighbors
  
  
        W = (D1 <=2) - eye(n);
        D = D1;
  
    Wbar = W ./(max(sum(W),.1)'*ones(1,n));   
       
    for thetaindex = 1:length(thetaset);
        theta = thetaset(thetaindex);
    [n,theta]
    fix(clock)
     
  covmatrix = theta .^D0;
        
        csmatrix = chol(covmatrix,'lower');
    
    % initilaize variables
        
    tstat_iid = zeros(nbrep,1); tstat_cshac = zeros(nbrep,1); tstat_white = zeros(nbrep,1);optband = zeros(nbrep,1);bootpvalue = zeros(nbrep,1);
    notpsd = zeros(nbrep,1);    nbnegative = zeros(nbrep,1); measure = nbnegative; J = zeros(nbrep,1); Jplus = J;bootpvaluep = bootpvalue;bhat = zeros(nbrep,1);
            J_diff_mean = zeros(nbrep,1); J_diff_var = J_diff_mean;

    for rep = 1:nbrep;
    
        if floor(rep/1000) == rep/1000
             rep  % print replication number every 1000th replication           
         end   
     
        % generate data
        
        epsilon = randn(n,2);
        z = csmatrix*epsilon;
        
        x = z(:,1); u = z(:,2);
        
        y = beta0*x + u;
        
        % project off constant - FWL theorem
        
        xdot = x - mean(x);
        ydot = y - mean(y);
        
        [bols,stb,r2,s2] = ols(ydot,xdot,0);   % regress y on preojected x
bhat(rep) = bols;        
        resid= ydot - bols*xdot;
        score = xdot .* resid;
        A = inv(xdot'*xdot/n);
        
        tstat_iid(rep) = (bols-beta0)/stb; % usual t statistic
        
         
% comppute "autocorrelation fuction"

d0 = .1*n^(1/6);
for bindex = 1:length(bandset);
    d = bandset(bindex)*n^(1/6);
for i = 1:n-1
    uij = resid(i) .* resid(i+1:n);
    kij = abs(D(i+1:n,i)-d)<d0;
    denom(i) = sum(kij);
    num(i) = sum(kij .*uij);
end %i
mhat(bindex) = sum(num)/sum(denom);
end %bindex

%boostrap

for b = 1:399

            rowsindex = randsample(n,n,1);
         
              score_iid = resid(rowsindex);
for bindex = 1:length(bandset);
    d = bandset(bindex)*n^(1/6);
for i = 1:n-1
    uij_star = score_iid(i) .* score_iid(i+1:n);
    kij = (D(i+1:n,i)-d)<d0;
    denom_star(i) = sum(kij);
    num_star(i) = sum(kij .*uij_star);
end %i
mhat_star(bindex,b) = sum(num_star)/sum(denom_star);
end %bindex
   
end % b

sig = mhat > quantile(mhat_star',.975);

if max(sig)>0;
[minsig,ind]=min(sig);
else
    ind = 1;
end

dn_hat = bandset(ind)*n^(1/6);

Dstar = D/dn_hat;
           
           K = exp(-Dstar.^2);
           cross = score*score';
           J_hat = sum(sum(K .*cross))/n;
           stb_cshac =sqrt (A*J_hat*A'/n);
           
           tstat_cshac(rep) = (bols-beta0)/stb_cshac;
                          
     J_white = sum(sum(cross .*eye(n)))/n;
     stb_white = sqrt(A*J_white*A'/n);
     tstat_white(rep) = (bols - beta0)/stb_white;
     
     % compute restricted residuals
     
     alphar = mean(y-beta0*x); % restricted estimator
     residr = y - alphar - beta0*x;
     
  Kstar =  K;

     [v2,d2] = eig(Kstar);
     
     d2star = max(real(d2),zeros(n,n));
     L = v2*sqrt(d2star);
     

     % bootstrap
     
   %  initialize variables
   tstat_cshac_star = zeros(B,1); tstat_cshac_star_iid = zeros(B,1); bhatstar = zeros(nbrep,1);J_boot = zeros(B,1);

     parfor nb = 1:B
         
         nu = randn(n,1); %independent N(0,1)
          eta = L*nu;
          estar = eta .* residr;
          
          ystar = alphar + beta0*x +estar;
         ystardot = ystar - mean(ystar);
         
        [bstar,stbstar,r2star,s2star] = ols(ystardot,xdot,0);   % regress y on preojected x
        bhatstar(nb) = bstar;

        residstar= ystardot - bstar*xdot;
        scorestar = xdot .* residstar;
                
        tstat_iid_star(nb) = (bstar-beta0)/stbstar; % usual t statistic
        
           crossstar = scorestar*scorestar';
           J_hatstar = sum(sum(K .*crossstar))/n;
           stb_cshacstar =sqrt (A*J_hatstar*A'/n);
               
           tstat_cshac_star(nb) = (bstar-beta0)/stb_cshacstar;


              
% iid boot (BCHV)
         
             rowsindex = randsample(n,n,1);
         
              ystar_iid = y(rowsindex);
              xstar_iid = x(rowsindex,:);

             ydotstar_iid = ystar_iid - mean(ystar_iid);
             xdotstar_iid = xstar_iid - mean(xstar_iid);
         
        [bstar_iid,stbstar,r2star,s2star] = ols(ydotstar_iid,xdotstar_iid,0);   % regress y on preojected x
        
        residstar_iid= ydotstar_iid - bstar_iid*xdotstar_iid;
        scorestar_iid = xdotstar_iid .* residstar_iid;
                
           crossstar_iid = scorestar_iid*scorestar_iid';
           J_hatstar_iid = sum(sum(K .*crossstar_iid))/n;
           Astar_iid = inv(xdotstar_iid'*xdotstar_iid/n);
           stb_cshacstar_iid =sqrt (Astar_iid*J_hatstar_iid*Astar_iid'/n);
           
           tstat_cshac_star_iid(nb) = (bstar_iid-bols)/stb_cshacstar_iid;
         
     end %nb
   
bootpvalue(rep,1) = sum(abs(tstat_cshac_star_iid)>abs(tstat_cshac(rep)))/B;
bootpvalue(rep,2) = sum(abs(tstat_cshac_star)>abs(tstat_cshac(rep)))/B;
 bootpvaluep(rep,2) = sum(abs((bhatstar)-bhat(rep))<abs(bhat(rep)-beta0))/B;


        end;   %rep
        

 display('Rejection rate of asymptotic tests (LS) for alpha - standard, CS-HAC, White)');
  [mean(abs(tstat_iid) >1.96),mean(abs(tstat_cshac) >1.96),mean(abs(tstat_white)>1.96),mean(bootpvalue <.05)]'*100        

    end % %theta
    
end % n